@@ -27,18 +27,22 @@
 
 #include <Lum/Def/Menu.h>
 #include <Lum/Def/MultiView.h>
+#include <Lum/Def/Props.h>
 
 #include <Lum/Dlg/About.h>
 #include <Lum/Dlg/Msg.h>
+#include <Lum/Dlg/Properties.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/Boolean.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 #include <Lum/Model/String.h>
 #include <Lum/Model/Table.h>
 
-#include <Lum/OS/Main.h>
+#include <Lum/Manager/Behaviour.h>
+#include <Lum/Manager/Display.h>
 
+#include <Lum/Application.h>
 #include <Lum/Boolean.h>
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
@@ -61,7 +65,6 @@
 #endif
 
 #include "Configuration.h"
-#include "ConfigDialog.h"
 
 #define NETWORK_REFRESH    10
 #define POLL_ERROR_TIMEOUT 60
@@ -72,20 +75,17 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdTable<Status::Network> NetworkModel;
 typedef Lum::Base::Reference<NetworkModel>    NetworkModelRef;
 
-class NetworkObject : public Lum::Object
+class NetworkModelPainter : public Lum::TableCellPainter
 {
 private:
-  Lum::OS::FontRef  font;
   Lum::OS::ColorRef red;
   Lum::OS::ColorRef green;
   Lum::OS::ColorRef yellow;
   Lum::OS::ColorRef grey;
-  Status::Network   network;
 
 public:
-  NetworkObject()
-  : font(Lum::OS::display->GetFont()),
-    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+  NetworkModelPainter()
+  : red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
     green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
     yellow(1,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
     grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
@@ -93,39 +93,19 @@ public:
     // no code
   }
 
-  void SetNetwork(const Status::Network& network)
+  size_t GetProposedHeight() const
   {
-    this->network=network;
+    return Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+           GetFont()->height+
+           Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+           GetFont()->height+
+           Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
   }
 
-  void CalcSize()
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    minWidth=font->height+
-             Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
-             3*font->StringWidth(L"m");
-    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-              font->height+
-              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-              font->height+
-              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
+    Status::Network network=dynamic_cast<const NetworkModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Network security type
@@ -152,19 +132,22 @@ public:
       draw->PushForeground(grey);
     }
 
-    draw->FillRectangle(this->x,
-                        this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
-                        font->height,font->height);
+    draw->FillRectangle(x,
+                        y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
+                        GetFont()->height,GetFont()->height);
     draw->PopForeground();
 
     //
     // Name of network
     //
 
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
-    draw->DrawString(this->x+font->height+Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder),
-                     this->y+font->ascent,
+    draw->PushFont(GetFont());
+    draw->PushForeground(GetTextColor(draw));
+    draw->DrawString(x+GetFont()->height+
+                     Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder),
+                     y+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     GetFont()->ascent,
                      network.essid);
     draw->PopForeground();
     draw->PopFont();
@@ -176,10 +159,12 @@ public:
     if (network.channel>=0) {
       std::wstring channel=L"Channel: "+Lum::Base::NumberToWString(network.channel);
 
-      draw->PushFont(font);
-      draw->PushForeground(parent->GetTextColor(draw));
-      draw->DrawString(this->x+width-font->StringWidth(channel)+1,
-                       this->y+font->ascent,
+      draw->PushFont(GetFont());
+      draw->PushForeground(GetTextColor(draw));
+      draw->DrawString(x+width-GetFont()->StringWidth(channel)+1,
+                       y+
+                       Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                       GetFont()->ascent,
                        channel);
       draw->PopForeground();
       draw->PopFont();
@@ -198,85 +183,33 @@ public:
       draw->PushForeground(green);
     }
 
-    draw->FillRectangle(this->x,
-                        this->y+
+    draw->FillRectangle(x,
+                        y+
                         Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-                        font->height+
+                        GetFont()->height+
                         Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
                         (size_t)(width*percent),height-2);
 
 
     draw->PopForeground();
 
-    Lum::OS::FontRef    font=Lum::OS::display->GetFont();
     std::wstring        label=Lum::Base::NumberToWString(lround(percent*100))+L"%";
     Lum::OS::FontExtent extent;
 
-    font->StringExtent(label,extent);
+    GetFont()->StringExtent(label,extent);
 
-    draw->PushForeground(Lum::OS::Display::textColor);
-    draw->PushFont(font,0);
-    draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
-                     this->y+
+    draw->PushForeground(GetTextColor(draw));
+    draw->PushFont(GetFont());
+    draw->DrawString(x+(width-extent.width+extent.left+extent.right)/2,
+                     y+
                      Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-                     font->height+
+                     GetFont()->height+
                      Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-                     font->ascent,
+                     GetFont()->ascent,
                      label);
     draw->PopFont();
     draw->PopForeground();
   }
-
-  static NetworkObject* Create(bool horizFlex, bool vertFlex)
-  {
-    NetworkObject *object=new NetworkObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class NetworkDataProvider : public NetworkModel::DataProvider
-{
-private:
-  NetworkObject *object;
-
-public:
-  NetworkDataProvider()
-  : object(NetworkObject::Create(true,false))
-  {
-    // no code
-  }
-
-  virtual ~NetworkDataProvider()
-  {
-    delete object;
-  }
-
-  /*
-  std::wstring GetString(const NetworkModel::Iterator& iter, size_t column) const
-  {
-    switch (column) {
-    case 1:
-      return iter->essid;
-    case 2:
-        if (iter->channel>=0) {
-          return Lum::Base::NumberToWString(iter->channel);
-        }
-        else {
-          return L"?";
-        }
-    default:
-      return L"";
-    }
-  } */
-
-  Lum::Object* GetObject(const NetworkModel::Iterator& iter, size_t column) const
-  {
-    object->SetNetwork(*iter);
-    return object;
-  }
 };
 
 class MyWindow : public Lum::Dialog
@@ -296,6 +229,7 @@ private:
   NetworkModelRef          tableModel;
 
   Lum::Model::BooleanRef   scanForNetworks;
+  Lum::Model::BooleanRef   signalScan;
   Lum::Model::BooleanRef   wardrive;
   Lum::Model::ActionRef    powerSavingOnAction;
   Lum::Model::ActionRef    powerSavingOffAction;
@@ -307,6 +241,7 @@ private:
 
   Lum::Model::ActionRef    configAction;
   Lum::Model::ActionRef    aboutAction;
+  Lum::Model::ActionRef    displayAction;
 
 #if defined(CAN_USE_STATUS_HILDON)
     StatusHildon           status;
@@ -334,11 +269,11 @@ private:
   {
     size_t i=0;
 
-    while (i<Lum::OS::display->GetArgCount()) {
-      if (Lum::OS::display->GetArg(i)==L"-d" &&
-          i+1<Lum::OS::display->GetArgCount()) {
+    while (i<Lum::Application::Instance()->GetArgumentCount()) {
+      if (Lum::Application::Instance()->GetArgument(i)==L"-d" &&
+          i+1<Lum::Application::Instance()->GetArgumentCount()) {
         i++;
-        status.SetDefaultInterface(Lum::Base::WStringToString(Lum::OS::display->GetArg(i)));
+        status.SetDefaultInterface(Lum::Base::WStringToString(Lum::Application::Instance()->GetArgument(i)));
       }
 
       i++;
@@ -376,7 +311,7 @@ private:
     CancelNetworks();
 
     if (status.SupportsNetworkRetrieval()) {
-      if (scanNetworkSound!=NULL) {
+      if (scanNetworkSound!=NULL && signalScan->IsEnabled() && signalScan->Get()) {
         std::cout << "*** Playing scan sound..." << std::endl;
         scanNetworkSound->Play();
       }
@@ -476,7 +411,10 @@ private:
       quality->SetNull();
     }
 
-    signalNoise->Set(Lum::Base::NumberToWString(status.signal-0x100)+L" dBm / "+Lum::Base::NumberToWString(status.noise-0x100)+L" dBm");
+    signalNoise->Set(Lum::Base::NumberToWString(status.signal-0x100)+
+                     L" dBm / "+
+                     Lum::Base::NumberToWString(status.noise-0x100)+
+                     L" dBm");
 
     switch (status.powerSaving) {
     case Status::powerSavingUnknown:
@@ -580,8 +518,9 @@ public:
     channel(new Lum::Model::String()),
     powerSaving(new Lum::Model::Boolean()),
     refreshTime(new Lum::Model::Int(5)),
-    tableModel(new NetworkModel(new NetworkDataProvider())),
+    tableModel(new NetworkModel()),
     scanForNetworks(new Lum::Model::Boolean()),
+    signalScan(new Lum::Model::Boolean(false)),
     wardrive(new Lum::Model::Boolean(false)),
     powerSavingOnAction(new Lum::Model::Action()),
     powerSavingOffAction(new Lum::Model::Action()),
@@ -591,6 +530,7 @@ public:
     networksErrorTimer(new Lum::Model::Action()),
     configAction(new Lum::Model::Action()),
     aboutAction(new Lum::Model::Action()),
+    displayAction(NULL),
     scanNetworkSound(NULL),
     openNetworkSound(NULL)
   {
@@ -598,9 +538,11 @@ public:
     refreshTime->SetRange(0,10);
 
     powerSaving->Disable();
+    signalScan->Disable();
     wardrive->Disable();
 
     Observe(scanForNetworks);
+    Observe(signalScan);
     Observe(wardrive);
     Observe(powerSavingOnAction);
     Observe(powerSavingOffAction);
@@ -615,6 +557,11 @@ public:
     Observe(configAction);
     Observe(aboutAction);
 
+    if (Lum::Manager::Display::Instance()!=NULL) {
+      displayAction=Lum::Manager::Display::Instance()	->GetDisplayStateChangeAction();
+      Observe(displayAction);
+    }
+
     Observe(status.statusChangedAction);
     Observe(status.networksChangedAction);
 
@@ -739,6 +686,8 @@ public:
       Lum::Model::SingleLineSelectionRef selection;
       Lum::Def::Boolean                  scanNetworksDef(Lum::Def::Desc(L"Scan for networks"),
                                                          scanForNetworks);
+      Lum::Def::Boolean                  signalScanDef(Lum::Def::Desc(L"Signal each scan"),
+                                                       signalScan);
       Lum::Def::Boolean                  wardriveDef(Lum::Def::Desc(L"Signal open networks"),
                                                      wardrive);
 
@@ -746,11 +695,15 @@ public:
       panel->Add(Lum::HPanel::Create(true,false)
                  ->Add(Lum::Label::Create(true,false)
                         ->AddLabel(L"Scan for networks:",
-                                   Lum::OS::display->GetBehaviour()->GetBooleanControl(scanNetworksDef,true,false)))
+                                   Lum::Manager::Behaviour::Instance()->GetBooleanControl(scanNetworksDef,true,false)))
+                        ->AddSpace()
+                        ->Add(Lum::Label::Create(true,false)
+                              ->AddLabel(L"SignalScan:",
+                                         Lum::Manager::Behaviour::Instance()->GetBooleanControl(signalScanDef,true,false)))
                         ->AddSpace()
                         ->Add(Lum::Label::Create(true,false)
                               ->AddLabel(L"Wardrive:",
-                                         Lum::OS::display->GetBehaviour()->GetBooleanControl(wardriveDef,true,false))));
+                                         Lum::Manager::Behaviour::Instance()->GetBooleanControl(wardriveDef,true,false))));
       panel->AddSpace();
 
       selection=new Lum::Model::SingleLineSelection;
@@ -770,6 +723,7 @@ public:
       //table->SetSelectionAction(showDetail);
       //table->SetDoubleClickAction(showDetail);
       table->SetShowHeader(false);
+      table->SetPainter(new NetworkModelPainter());
       table->GetTableView()->SetMinHeight(Lum::Base::Size::stdCharHeight,4);
       table->GetTableView()->SetAutoFitColumns(true);
       table->GetTableView()->SetAutoHSize(true);
@@ -801,7 +755,7 @@ public:
 
     multiView.SetMenu(preMenu,postMenu);
 
-    Lum::OS::display->GetBehaviour()->ApplyMultiViewDlg(this,multiView);
+    Lum::Manager::Behaviour::Instance()->ApplyMultiViewDlg(this,multiView);
 
     Dialog::PreInit();
   }
@@ -835,12 +789,14 @@ public:
           *scanForNetworks) {
         tableModel->SetEmptyText(L"(No networks found)");
         RequestNetworks();
+        signalScan->Enable();
         wardrive->Enable();
       }
       else {
         tableModel->SetEmptyText(L"(Scanning not enabled)");
         CancelNetworks();
         UpdateNetworks();
+        signalScan->Disable();
         wardrive->Disable();
       }
     }
@@ -877,29 +833,47 @@ public:
       Lum::OS::display->AddTimer(NETWORK_REFRESH,0,networksTimer);
     }
     else if (model==configAction && configAction->IsFinished()) {
-      ConfigDialog *dialog;
+      Lum::Def::PropGroup *props=new Lum::Def::PropGroup();
+
+      props->Int(Lum::Def::Int(Lum::Def::Desc(L"Refresh time"), refreshTime));
 
       CancelStatus();
-      dialog=new ConfigDialog(refreshTime);
-      dialog->SetParent(this);
-      if (dialog->Open()) {
-        dialog->EventLoop();
-        dialog->Close();
-      }
+
+      Lum::Dlg::Properties::Show(this,props);
+
+      delete props;
+
       RequestStatus();
     }
     else if (model==aboutAction && aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
     }
+    else if (displayAction.Valid() &&
+             model==displayAction) {
+      Lum::Manager::Display::DisplayState state=Lum::Manager::Display::Instance()->GetDisplayState();
+      if (state==Lum::Manager::Display::DisplayStateOn) {
+        RequestStatus();
+        if (!scanForNetworks->IsNull() && *scanForNetworks) {
+          RequestNetworks();
+        }
+      }
+      else if (state==Lum::Manager::Display::DisplayStateOff) {
+        // unless wardriving (which is audible), disable everything else.
+        if (wardrive->IsNull() || !wardrive->IsEnabled() || !*wardrive) {
+          CancelStatus();
+          CancelNetworks();
+        }
+      }
+    }
 
     Dialog::Resync(model,msg);
   }
 };
 
-class Main : public Lum::OS::MainDialog<MyWindow>
+class Main : public Lum::GUIApplication<MyWindow>
 {
 public:
-  Main()
+  bool Initialize()
   {
     info.SetProgram(Lum::Base::StringToWString(PACKAGE_NAME));
     info.SetVersion(Lum::Base::StringToWString(PACKAGE_VERSION));
@@ -908,6 +882,8 @@ public:
     info.SetContact(L"Tim Teulings <tim@teulings.org>");
     info.SetCopyright(L"(c) 2004, Tim Teulings");
     info.SetLicense(L"GNU Public License");
+
+    return Lum::GUIApplication<MyWindow>::Initialize();
   }
 };
 
